#pragma once

#include "device_iface.h"
#include "rate_utils.h"
#include <list>
#include <memory>
#include <libusb-1.0/libusb.h>

class OmniBuffer {
  struct libusb_transfer* xfer;
  std::vector<uint8_t> _data;
  bool pending;
  static void __transfer_done(struct libusb_transfer* xfer0);
  void transfer_done(struct libusb_transfer* xfer0);
public:
  OmniBuffer(libusb_device_handle* handle,
	     int size = 8*1024,
	     int endpoint = 0x81);
  ~OmniBuffer();
  void submit();
  bool isPending() const;
  const uint8_t* data() const;
  uint8_t* data();
  int size() const;
};

class OmniParseFIFO {
  struct BufferInfo {
    std::shared_ptr<OmniBuffer> buf;
    int pos8;
    int remaining8;
  };
  std::vector<BufferInfo> bufs;
public:
  int remaining();

  void consume(void* dst, int n);
  uint32_t consume32();

  void push(std::shared_ptr<OmniBuffer> buf);
  void clear();
  void flush(std::list<std::shared_ptr<OmniBuffer> >& buffers);
};

class OmniDevice : public OccamDeviceBase {
  std::string cid;
  libusb_device_handle* handle;
  std::list<std::shared_ptr<OmniBuffer> > buffers;
  OmniParseFIFO stream;
  int buffer_count;
  int max_buffers;
  DataRateCounter xfer_rate;
  FrameCounter frame_rate;
  OccamImage* image_fr;
  int pixels_pos;
  int pixels_size;
  int frame_count;

  int max_queued_frames;
  int queued_frames_count;
  std::list<OccamImage*> queued_frames;

  int state;
  uint32_t addr16;
  uint32_t addr16_noflags;
  int sensor_index;
  int pixel_addr8;

  void newDataAvailable(std::shared_ptr<OmniBuffer> buf);
  void initFrame();
  void emitFrame(OccamImage* image_fr);

public:
  OmniDevice(const std::string& cid, int max_buffers = 32);
  ~OmniDevice();
  bool isOpen() const;

  virtual int writeRegister(uint32_t addr, uint32_t value);
  virtual int readRegister(uint32_t addr, uint32_t* value);

  virtual int writeStorage(uint32_t addr, uint32_t len, const uint8_t* data);
  virtual int readStorage(uint32_t addr, uint32_t len, uint8_t* data);

  virtual int readImage(OccamImage** image, int block);

  void update();
};


// Local Variables:
// mode: c++
// End:
