#include "rate_utils.h"

//////////////////////////////////////////////////////////////////////////////////
// FrameCounter

FrameCounter::FrameCounter()
  : last_time(0), count(0), last_rate(0) {
}

bool FrameCounter::increment() {
  bool r = false;
  time_t now = time(0);
  if (now != last_time) {
    last_rate = count;
    last_time = now;
    count = 0;
    r = true;
  }
  ++count;
  return r;
}

int FrameCounter::rate() const {
  return last_rate;
}

//////////////////////////////////////////////////////////////////////////////////
// DataRateCounter

DataRateCounter::DataRateCounter()
  : last_byte_count(0),
    total_bytes(0) {
}

bool DataRateCounter::increment(int bytes) {
  total_bytes += bytes;
  bool r = false;
  byte_count += bytes;
  time_t now = time(0);
  if (last_time != now) {
    last_byte_count = byte_count;
    last_time = now;
    byte_count = 0;
    r = true;
  }
  return r;
}

double DataRateCounter::rateMBs() const {
  return last_byte_count / 1024. / 1024.;
}

double DataRateCounter::totalMB() const {
  return double(total_bytes) / 1024 / 1024;
}

uint64_t DataRateCounter::totalBytes() const {
  return total_bytes;
}
